/*               Griddy-Gibbs for GARCH-Student - Simulated data 

L. Bauwens and M. Lubrano "Bayesian inference on GARCH models using the Gibbs sampler"

This program produces the output needed to draw the graphs and plots of the paper, 
in Figures 1, 2 and 3, using of course the appropriate input files that have to be loaded
and which are produced by GG.PRG using the appropriate options. In particular, for 
Figure 2, see the comments belox in the program (lines 161-163).

This program is a follow-up of GG.PRG (which generates the basic output). It must be run
after running GG.PRG, to get additional output: the posterior results (same as in the 
output of GG.PRG), plus graphs of marginal densities, CUMSUM plots, autocorrelation 
functions of the draws... It prints all the output necessary to draw the graphs of 
the paper (the graphs were produced with GNUPLOT except for Figure 2, which are directly the GAUSS graphs, 
converted to EPS type). Additional output could be computed at your discretion, of course.

It is ***CRUCIAL*** that the options of this program are made consistent with those GG.PRG 
when it is run: this means that
- TETA_INF and TETA_SUP must be exactly as they are set in GG.PRG. See lines 129-130 below.
- NDROP must also be set at the same value as in GG.PRG. See line 142. Notice that the
  value of N (total number of draws) is deduced from the data (see line 150).
- the names of the files produced by GG.PRG which are loaded in this program must of course
  be as they were in GG.PRG (TIR150AF, TES150AF, TVA150AF, TVA150AF, and MAR150AF in this
  version). See lines 147 to 149, and line 201.
-the message printing which prior on nu is used should be adjusted (if not, there is no
 harmful consequence on the results, only a wrong prior name). See lines 153-156.

This program also redoes the maximum likelihood estimation (using PROC LFGARCH).
Don't forget also the OUTPUT and SPATH instructions (lines 33 and 35).
*/ 

library pgraph,maxlik;
OUTPUT file = c:\mydoc\articles\gibbs\calculs\ggf150M.ou3 reset;
graphset;
fonts ("simplex complex microb simgrma");
SPATH="c:\\mydoc\\articles\\gibbs\\calculs"; @path for saving results@
datestr(0);;" at ";;timestr(0);
__title = "Artificial data (Student) - Griddy Gibbs sampling";
print $__title;
format 8,4;


@@@ Generate data: DGP is GARCH(1,1) @@@

rndseed 13579;
nt = 151;
yt = zeros(nt,1);
alpha = 0.4;
beta =  0.4;
nu = 5;
omega = 0.1;

y = ones(nt,1);
u = zeros(nt,1);
ht = ones(nt,1);
dep = 1; @starting value for evaluation of ht in l.f.@

i = 2; do while i<=nt;
    ht[i] = omega + alpha*u[i-1]^2 + beta*ht[i-1];
    u[i] = studt(0,sqrt(ht[i]),nu,1);
    y[i] = u[i];
i = i+1; endo;

"omega = ";; omega;;"alpha = ";;alpha;;"   beta = ";;beta;;"   nu = ";;nu;
@"rho = ";;rho;;"   mu = ";;mu;@
if nu > 2;
   wsc = 1 - alpha*nu/(nu-2) - beta;
   "weak stationarity condition ";;wsc;
   if wsc > 0;
      uv = omega/wsc;
      "unconditional variance ";; uv;
   else;
      "unconditional variance does not exist";
   endif;
else;
   "nu smaller than 2";
endif;
@
var = cons;
xy(seqa(1,1,nt),y~sqrt(ht));
@

stdy = stdc(y);
"EcT initial de la serie = ";;stdy;
y = y/stdy;
dep = dep/stdy;

yt = y[2:nt];
y1 = y[1:nt-1];
x = yt;
uq = yt^2;
t = rows(yt);
vy = vcx(yt);
@save path=^spath y;@
@"y";;y;@
"";"number of observations";;t;
"mean and s.d.";;meanc(y)~sqrt(vy);


@@@ Maximum likelihood estimation with MAXLIK @@@

#include maxlik.ext;
maxset;
__output = 50;
_mlparnm = "o_root" | "a_root" | "g_root" | "nu   ";
@ GARCH parameters are square roots of original ones, see lfgarch: @
teta_i = sqrt(omega)|sqrt(alpha)|sqrt(beta)|nu;
@starting values for m@
"";"teta_i";;teta_i';
{teta_h,l,s,cov,retcode} = maxprt(maxlik(yt,0,&lfgarch,teta_i));
teta_ho = teta_h[1:3]^2 | teta_h[4] ;
save path=^spath teta_ho;
jc = zeros(4,4);
jd =  2*teta_h[1:3] | 1; @jacobian (diagonal)@
jc = diagrv(jc,jd);
covq = jc*cov*jc;
"";"MLE of natural parameters:";;
teta_se = sqrt(diag(covq));
"";"teta_h se";;teta_ho~teta_se;
"";"correlation matrix";;corrvc(covq);


@@@ Posterior results by Gibbs sampling  @@@
          
          @ initialisations @

name = "\204w"~"\204a"~"\204b"~"\204n"; @parameter names@
k = cols(name);                    @number of parameters@
teta_inf={0.001 0.001 0.001 0.001};@lower bounds of integration ranges@
teta_sup={1.00   1.00   1.00   30};@upper bounds of integration ranges@
M=33;                              @number of points for marginals@
step = (teta_sup-teta_inf)/(m-1);  @increments@
teta_gr = zeros(m,k);              @abcissae of each parameter for Simpson@
i=1;
do while i<m+1;
  teta_gr[i,.] = teta_inf + (i-1)*step;
  i=i+1;
endo; 

@load teta_ho = c:\mydoc\articles\gibbs\calculs\teta_ho.fmt;@
TETA_0 = teta_ho;                  @initial conditioning values of Gibbs@
NDROP = 1000;                      @# left out initial draws for final results@


          @ read table of draws @
  
load teta = c:\mydoc\articles\gibbs\calculs\TIRA150AF.fmt;
load cexp = c:\mydoc\articles\gibbs\calculs\TESC150AF.fmt;
load cvar = c:\mydoc\articles\gibbs\calculs\TVAC150AF.fmt;
N = rows(teta);
"";"BAYESIAN RESULTS BY GIBBS SAMPLING:";
"";"number of rows and columns of teta:";;rows(teta)~cols(teta);

@ CHOOSE THE APPROPRIATE MESSAGE ON THE PRIOR @
"Prior on nu: flat on nu";
@"Prior on nu: flat on 1/nu";@
@"Prior on nu: half Cauchy on nu";@


/* 
Figure 2 in the paper was produced with the following commented out section, using the
right imput files (not TIRA150AF.fmt, but TIRA150M.fmt which is the file of draws
I generated for a half-Cauchy prior on nu on (0,18), N=10000 and NDROP=1000)
*/
/*
_pdate="";
_plctrl = {-1};
_pstype = 4;
@title("Scatter plot of 1000 draws of alpha and beta");@
xtics(0,1,0.1,1);
ytics(0,1,0.1,1);
title("2-a: 100 draws");
xlabel("alpha"); ylabel("beta");
xy(teta[ndrop+1:1100,2],teta[ndrop+1:1100,3]);
title("2-b: 500 draws");
xy(teta[ndrop+1:1500,2],teta[ndrop+1:1500,3]);
title("2-c: 1000 draws");
xy(teta[ndrop+1:2000,2],teta[ndrop+1:2000,3]);
title("2-d: 10000 draws");
xy(teta[ndrop+1:n,2],teta[ndrop+1:n,3]);
*/


   @@@ final results @@@
 
pomean = meanc(teta[ndrop+1:n,.]);
povcov = vcx(teta[ndrop+1:n,.]);
posd = sqrt(diag(povcov));
pamean = meanc(cexp[ndrop+1:n,.]);
pavcov = meanc(cvar[ndrop+1:n,.]) + diag(vcx(cexp[ndrop+1:n,.]));
pasd = sqrt(pavcov);
"";"Posterior results by Gibbs:";
"initial value and integration bounds";;teta_0~teta_inf'~teta_sup';
"Trapezoidal rule with m (for densities)";;m;

nj = n-ndrop;
"number of draws used for results";;nj;
"posterior means and st. dev.";;pomean~pamean~posd~pasd;
"posterior correlation matrix";;corrvc(povcov);

load marg = c:\mydoc\articles\gibbs\calculs\MAR150AF.fmt;
marg = marg/nj;
i=1;
do while i<k+1;
  @window(2,2,0);
  setwind(1);
  title("Histogram appr. of post. density of "$+name[i]);
  {b,mid,freq} = histp(teta[ndrop+1:n,i],17);
  setwind(3);@
  title("Posterior density of "$+name[i]);
  xy(teta_gr[.,i],marg[.,i]);
  @endwind;@
  "";"parameter";;i;
  "abs dens";;teta_gr[.,i]~marg[.,i];
   i=i+1;
endo;



          @@@ convergence diagnostics @@@

incn = 50; @value of 50 can be changed to 20, 10, or 100...@
jmax = (n-ndrop)/incn;
bande = zeros(jmax,2);
bande[.,1] = 0.05*ones(jmax,1); bande[.,2] = -0.05*ones(jmax,1);
index = seqa(1,1,jmax);
exconv = zeros(jmax,k);
ecconv = zeros(jmax,k);
@sdconv = zeros(jmax,k);@
j=1;
do while j<jmax+1;
  nt = j*incn;
  nj = ndrop + nt;  
  exconv[j,.] = ((meanc(teta[ndrop+1:nj,.]) - pomean)./posd)';
  ecconv[j,.] = ((meanc(cexp[ndrop+1:nj,.]) - pamean)./pasd)';
@  sdconv[j,.] = sqrt(diag(vcx(teta[ndrop+1:nj,.])))';@
  j=j+1;
endo;
nac = 50; iac = seqa(0,1,nac+1);
acf = autocorr(teta[ndrop+1:n,.],nac);
ccf = autocorr(cexp[ndrop+1:n,.],nac);
j=1;
do while j<k+1;
@  window(2,2,0);
  setwind(1); @
  title("Convergence graph of mean estimates of "$+name[j]);
  xy(index,exconv[.,j]~ecconv[.,j]);
  "";"CUMSUM series for parameter";;j;index~exconv[.,j]~ecconv[.,j]~bande;
@  setwind(2);
  title("Convergence graph of c. exp. mean of "$+name[j]);
  xy(index,ecconv[.,j]);
  title("Convergence graph of s.d. of "$+name[j]);
  xy(index,sdconv[.,j]);
  setwind(3);
  title("Convergence graph of mean/s.d. of "$+name[j]);
  xy(index,exconv[.,j]./sdconv[.,j]);
  setwind(3); 
  title("Autocorrelogram of draws of "$+name[j]);
  bar(iac,acf[.,j]);
  setwind(4);
  title("Autocorrelogram of c. exp. of "$+name[j]);
  bar(iac,ccf[.,j]);
  endwind; @
  "";"autocorrelations of draws and c. exp. of";;j;;iac~acf[.,j]~ccf[.,j];
  j=j+1;
endo;


end;


PROC AUTOCORR(x,p);
@ compute p autocorrelation coefficients for each column of x
  result in a (p+1 by cols(x) matrix) @
local n,k,j,i,xi,xl,r,ac;
n = rows(x);
k = cols(x);
xi = zeros(n,2);
ac = zeros(p+1,k);
j = 1;
do while j < k+1;
  xi[.,1] = x[.,j];
  ac[1,j] = 1;
  i = 1;
  do while i < p+1;
     xi[.,2] = lagn(xi[.,1],i);
     xl = trimr(xi,i,0);
     r = corrx(xl);
     ac[i+1,j] = r[1,2];
     i = i+1;
  endo;  
  j = j+1;
endo;
retp (ac);
endp;


proc LFGARCH(teta,x);
@* likelihood evaluation at teta for each data in x
   Student distribution, for MAXLIK
   result is a vector (value of LF for each row of x) *@
   local v,nu,nu1h,nuv;
   v = recserar(teta[1]^2+teta[2]^2*lag1(uq),vy,teta[3]^2);
  nu = teta[4]; nu1h = (nu+1)/2; nuv = nu*v;
  retp ( ln(gamma(nu1h))-ln(gamma(nu/2))-0.5*ln(nuv)-nu1h*ln(1+uq./nuv) );
endp;


proc STUDT(bi,cholimi,nui,k);
@* Draw a Student vector of size k, mean bi and variance nu/(nu-2)*s*M^{-1} 
   NB: cholmi must be equal to chol(M^{-1})' (transpose is crucial) *@
retp( bi + cholimi*rndn(k,1)/sqrt(chi2(nui)/nui) );
endp;


proc CHI2(nu);
@* Chi-square algorithme in Rubinstein (1981) "Simulation and the Monte Carlo Method" 
   (Wiley),pages 93-94 *@
local k2,k2i;
if nu > 30;
   retp( (rndn(1,1)+sqrt(2*nu-1))^2/2 );
endif;
k2 = nu/2;
k2i = int(nu/2);
if k2 == k2i;
   retp( -2*ln(prodc(rndu(k2,1))) );
else;
   retp( -2*ln(prodc(rndu(k2,1))) + rndn(1,1)^2);
endif;
endp;