/*                    Metropolis for GARCH-Student - Simulated data 

This program is needed to reproduce the results for Tables 3 and 4 (Metropolis-Hastings
results) of the paper:
L. Bauwens and M. Lubrano "Bayesian inference on GARCH models using the Gibbs sampler"

This version of the program generates the results in Table 3, ie for 
-a half-Cauchy prior on nu on (0,18)
-intervals of integration of all parameters are assigned in lines 182-187 (a rejection rule)
-10000 draws are used in a single round (see lines 149-155)

This program must be slightly changed to reproduce the results in Table 4:
-change the prior on nu to flat: see PROC LRD (select the appropriate RETP instruction, 
 see lines 291-296). Also see lines 258-261 to write the selected prior in the output.
-adjust the upper limit of integration of nu by changing the rejection rule 
 (change 18 to 30 on line 183)
-see lines 124 to 134 (tuning of variances of importance function)
-also change the output file name mh150 (line 25), and the SPATH instruction (line 28).

Of course, there are other options which can be changed: see comments in the program.
This program does also the maximum likelihood estimation (using PROC LFGARCH)
*/ 

library pgraph,maxlik;
OUTPUT file = c:\mydoc\articles\gibbs\calculs\mh150.ou3 reset;
graphset;
fonts ("simplex complex microb simgrma");
SPATH="c:\\mydoc\\articles\\gibbs\\calculs"; @path for saving results@
datestr(0);;" at ";;timestr(0);
__title = "Artificial data (Student) - Metropolis-Hastings";
print $__title;
format 8,4;


@@@ Generate data: DGP is GARCH(1,1) @@@

rndseed 13579;
nt = 151;
yt = zeros(nt,1);
alpha = 0.4;
beta =  0.4;
nu = 5;
omega = 0.1;

y = ones(nt,1);
u = zeros(nt,1);
ht = ones(nt,1);
dep = 1; @starting value for evaluation of ht in l.f.@

i = 2; do while i<=nt;
    ht[i] = omega + alpha*u[i-1]^2 + beta*ht[i-1];
    u[i] = studt(0,sqrt(ht[i]),nu,1);
    y[i] = u[i];
i = i+1; endo;

"omega = ";; omega;;"alpha = ";;alpha;;"   beta = ";;beta;;"   nu = ";;nu;
@"rho = ";;rho;;"   mu = ";;mu;@
if nu > 2;
   wsc = 1 - alpha*nu/(nu-2) - beta;
   "weak stationarity condition ";;wsc;
   if wsc > 0;
      uv = omega/wsc;
      "unconditional variance ";; uv;
   else;
      "unconditional variance does not exist";
   endif;
else;
   "nu smaller than 2";
endif;
@
var = cons;
xy(seqa(1,1,nt),y~sqrt(ht));
@

stdy = stdc(y);
"EcT initial de la serie = ";;stdy;
y = y/stdy;
dep = dep/stdy;

yt = y[2:nt];
y1 = y[1:nt-1];
x = yt;
uq = yt^2;
t = rows(yt);
vy = vcx(yt);
@save path=^spath y;@
@"y";;y;@
"";"number of observations";;t;
"mean and s.d.";;meanc(y)~sqrt(vy);


@@@ Maximum likelihood estimation with MAXLIK @@@

#include maxlik.ext;
maxset;
__output = 50;
_mlparnm = "o_root" | "a_root" | "g_root" | "nu   ";
@ GARCH parameters are square roots of original ones, see lfgarch: @
teta_i = sqrt(omega)|sqrt(alpha)|sqrt(beta)|nu;
@starting values for m@
"";"teta_i";;teta_i';
{teta_h,l,s,cov,retcode} = maxprt(maxlik(yt,0,&lfgarch,teta_i));
teta_ho = teta_h[1:3]^2 | teta_h[4] ;
save path=^spath teta_ho;
jc = zeros(4,4);
jd =  2*teta_h[1:3] | 1; @jacobian (diagonal)@
jc = diagrv(jc,jd);
covq = jc*cov*jc;
"";"MLE of natural parameters:";;
teta_se = sqrt(diag(covq));
"";"teta_h se";;teta_ho~teta_se;
"";"correlation matrix";;corrvc(covq);


@@@ Metropolis-Hastings algorithm @@@

    @ Initialisation of importance function @

nui = 5;
"Degrees of freedom of imp. fct. ";;nui;
bi = teta_ho;
k = rows(bi);

@ TUNING of variance matrix of importance function:@
@ for half-Cauchy prior on nu (Table 3): inflation factor = 1.2 for all parameters:@
varb = 1.2^2*covq;

@ The previous statement must be replaced by the following three for Table 4 results
(flat prior on nu) since the inflation factor set to 4 for nu (see text of the paper):@
@
mid = eye(4);
mid[1,1] = 1.2;  mid[2,2] = 1.2; mid[3,3] = 1.2;  mid[4,4] = 4;
varb = mid*covq*mid;
@

ecb = sqrt(diag(varb));
"S.-D. of i.f.: ecb =  ";;ecb';
cholimi = chol(varb*(nui-2)/nui)';
mi = invpd(varb)*nui/(nui-2);

@initialisation for marginals@
grz = zeros(33,k);
nec = 4;
dgrx = ecb*2*nec/32;
i = 1; do while i <= k;
  grz[.,i] = seqa(bi[i]-nec*ecb[i],dgrx[i],33);
i = i+1;endo;

@ number of draws:
  For two rounds (ie updating of import. function: use NTRI = 2, and NTI = {5000 10000} 
  for 5000 draws in first round and 10000 in second round @
@NTI = {50000 10000};@
NTI = 10000;
@number of re-initialisations of parameters of imp. fct. : 1, 2, ...@
NTRI = 1;


    @ loop for draws @

intri = 1; do while intri <= ntri;
@rdi = zeros(nti[intri],1);@
@initialisation of sums@
rds = 0;
bs = zeros(k,1);
bbs = zeros(k,k);
grf = zeros(33,k);
@mb = zeros(nti[intri],k);@

@ starting point of MH @
bp = bi;
rdfip = exp(lrd(bp)-lfi(bp));

nrj = 0; @ number of rejections for positivity @
nrm = 0; @ number of rejections of MH @
datestr(0);;" at ";;timestr(0);


  inti = 1; do while inti <= nti[intri];
    rejet:
    bn = studt(bi,cholimi,nui,k);
@    mb[inti,.] = b'; @
@ REJECTION RULE: @
        if sumc(bn[1:4] .< .001) > 0  OR  sumc(bn[1:3] > 1) > 0   OR  bn[4]>18;
@        ".";; @
        nrj = nrj+1;
        goto rejet;
    endif;
   
    rdfin = exp(lrd(bn)-lfi(bn));

    ratio = rdfin/rdfip; 
    if ratio >= 1;
       bt = bn;
    else;
       u = rndu(1,1);
       if u < ratio; 
            bt = bn;
       else;
            bt = bp;
@            "+";; @
            nrm = nrm + 1;
       endif;                         
    endif;

@    rdi[inti] = ratio;@
    bp = bt;
    rdfip = rdfin;
@    mb[inti,.] = bt';@

    bs = bs + bt;
    bbs = bbs + bt*bt';
    i = 1; do while i <= k;
          grf[.,i] = grf[.,i] + (grz[.,i] .> bt[i]);
    i = i+1; endo;
  inti = inti+1; endo;

  bi = bs/nti[intri];
  bbi = bbs/nti[intri];
  varb = bbi - bi*bi';


  " ";"Intermediate results";
  "Number of rejections for positivity= ";;nrj;
  "Number of rejections of MH= ";;nrm;
  "Number of draws = ";;nti[intri];
  "E(b|y) =  ";;bi';
   ecb = sqrt(diag(varb));
  "Sd(b|y) = ";;ecb';
  "Correlation matrix";
   corrvc(varb);

   cholimi = chol(varb*(nui-2)/nui)';
   mi = invpd(varb)*nui/(nui-2);


@ reinitialisation for marginals@
dgrx = ecb*2*nec/32;
i = 1; do while i <= k;
  grz[.,i] = seqa(bi[i]-nec*ecb[i],dgrx[i],33);
i = i+1;endo;

datestr(0);;" at ";;timestr(0);
intri = intri+1; endo;

/* var = cons;
grm = grf;
h = 0.001;
i = 1; do while i<=k;
    xm = grz[.,i];
    ym = grf[.,i];
    j = 1; do while j <= 33;
        xd = grz[j,i];
        grm[j,i] = maxc((polyint(xm,ym,xd+h) - polyint(xm,ym,xd-h))/(h*2)|0);
    j = j+1; endo;
    xy(grz[.,i],grm[.,i]);
i = i+1; endo; */

@ WRITE CHOSEN PRIOR FOR nu: @
@"Prior on nu: flat on nu";@
@"Prior on nu: flat on 1/nu";@
"Prior on nu: half Cauchy on nu";
@save bi,varb;@

end;


proc LFGARCH(teta,x);
@* likelihood evaluation at teta for each data in x
   Student distribution, for MAXLIK
   result is a vector (value of LF for each row of x) *@
   local v,nu,nu1h,nuv;
   v = recserar(teta[1]^2+teta[2]^2*lag1(uq),vy,teta[3]^2);
  nu = teta[4]; nu1h = (nu+1)/2; nuv = nu*v;
  retp ( ln(gamma(nu1h))-ln(gamma(nu/2))-0.5*ln(nuv)-nu1h*ln(1+uq./nuv) );
endp;


proc LRD(b);
@* log posterior of GARCH-Student and AR(1) on yt 
   with half-Cauchy prior on nu *@

  local alpha,beta,omega,nu,ht,nu1h,nuv,fct;
  omega = b[1];
  alpha = b[2];
  beta  = b[3];
  nu    = b[4];
  ht = recserar(omega+alpha*uq[1:nt-2],dep,beta);
  nu1h = (nu+1)/2;
  nuv = nu*ht;
  fct = ln(gamma(nu1h))-ln(gamma(nu/2))-0.5*ln(nuv)-nu1h*ln(1+uq[2:nt-1]./nuv);
@ flat prior on nu: @ 
@ retp ( sumc(fct) ); @
@ uniform prior on 1/nu:  @
@  retp ( sumc(fct) -2*ln(nu) ); @
@ half Cauchy prior on nu:  @
retp ( sumc(fct) -ln(1+nu^2) );
endp;


proc LFI(b);
@* log of Student importance function *@

  retp( -(nui+k)/2*ln(nui+(b-bi)'Mi*(b-bi)) );
endp;


proc STUDT(b,cholmi,nu,k);
@* draw of Student_k with mean b and variance nu/(nu-2)*M^{-1}
   cholmi must be equal to chol(M^{-1})' (transpose is crucial) *@

retp( b + cholmi*rndn(k,1)/sqrt(chi2(nu)/nu) );
endp;


proc CHI2(nu);
/* Algorithme dans Rubinstein page 93-94 */
local k2,k2i;

if nu > 30;
   retp( (rndn(1,1)+sqrt(2*nu-1))^2/2 );
endif;
k2 = nu/2;
k2i = int(nu/2);
if k2 == k2i;
   retp( -2*ln(prodc(rndu(k2,1))) );
else;
   retp( -2*ln(prodc(rndu(k2,1))) + rndn(1,1)^2);
endif;
endp;