/*
Program for computing S(0)/N where S(0) is the spectral density at zero 
and N the sample size (number of draws)
Three methods are used:
-Tukey-Hanning window
-Daniell window
-Truncated priodogram Window (with initial positive sequence estimator of Geyer)
For more details: refer to the paper 
L. Bauwens and Pierre Giot: A Gibbs sampling approach to cointegration, forthcoming in
Computational Statistics, 1998.

This version does the computation for the parameter \alpha^-, which corresponds to the 
seventh column in the file TESATHC1 which contains the values of the conditional 
expectations produced by the Griddy-Gibbs. The file TESATHC.fmt is an output of the 
program ATHC.PRG

To do the computations for each other parameter, change the program accordingly:
- in line 37, select the column of the parameter (1 for mu, 2 for rho, 3 for omega,
  4 for alpha^+, 5 for beta, 6 for nu, 7 for alpha^-)
- change the name of the output file (line 40) and of the parameter(line 41)
- adjust the parameter MTK (line 71) for the Tukey-Hanning window and the parameter 
  MD (line 88) for the Daniell window): 
  MTK is the number of autocorrelations which are non-zero (look at the autocorrelograms 
  produced by ATHCF.PRG for the conditional expectations). We used mtk = 10 for mu and rho, 
  60 for omega, beta and alpha^-, 15 for nu, 30 for alpha^+;
  MD is determined as 3.14 times w, where w is the value of the abcissa correponding to
  the half of the height of the estimated spectral density (look at the spectral density
  of the conditional means produced by ATHCF.PRG); if the spectral density is flat, take 
  w as 4 or whatever. We used w = 4 for mu, rho, and nu (so md = 12), w = 10 (hence md = 30)
  for the other parameters (i.e. omega, beta, alpha^+, and alpha^-)
*/

library pgraph;
graphset;
@ load file of conditional expectations generated by program ATHC.PRG: @
load tir = c:\mydoc\articles\gibbs\spotw\TESATHC1;
Y = TIR[1001:2000,7]; @uses retained draws (last 1000); 7 is for \alpha^- @
nmc = rows(y); @ Monte Carlo sample size@
t=1000; @ nunmber of autocovariances for spectral density estimation @
OUTPUT file=c:\mydoc\articles\gibbs\spotw\nvAMINUS.out reset;
"Paramtre A-";
"Monte Carlo sample size: N=";;nmc;

/*
*************************************************************************************************
*/

ym=meanc(y);
print "Mean";;ym;
nvar = vcx(y)/nmc;
print "St. deviation/sqrt(N)= " sqrt(nvar);
print "Mean=" ym;
j=0;
gj=zeros(1000,1);
@ gj(1)=gamma(0), gj(2)=gamma(1), ..., gj(t)=gamma(t-1) @
do while j<=t-1;
k=j+1;
a=0;
do while k<=nmc;
a=a+(y[k]-ym)*(y[k-j]-ym);
k=k+1;
endo;
gj[j+1,1]=a/nmc;
j=j+1;
endo;

/* Compute S(0)/nmc */

@ Tukey-Hanning Window @
s=0;
MTK=60;
print "Tukey-Hanning Window";
print "M=" mtk;
s=s+gj[0+1,1];
j=1;
do while j<=mtk;
s=s+2*gj[j+1,1]*0.5*(1+cos(pi*j/mtk));
j=j+1;
endo;
s=s/(2*pi);
varth = s/nmc;
"NSE and RNE= ";;sqrt(varth)~nvar/varth;
@NSE means Numerical Standard Error and RNE means Relative numerical efficiency@


@ Daniell Window @
s=0;
MD=30;
print "Daniell Window";
print "M=" md;
s=s+gj[0+1,1];
j=1;
do while j<=t-1;
s=s+2*gj[j+1,1]*((sin(pi*j/md))/(pi*j/md));
j=j+1;
endo;
s=s/(2*pi);
varda = s/nmc;
"NSE and RNE= ";;sqrt(varda)~nvar/varda;


@ Truncated perio. window @
@  initial positive sequence estimator from autocovariances @

j=0;
ggj=zeros(t/2,1);
do while j<(t/2)-1;
ggj[j+1,1]=gj[2*j+1,1]+gj[2*j+2,1];
j=j+1;
endo;

j=0;
do while ggj[j+1,1]>0;
j=j+1;
endo;

print "Truncated periodogram Window";
s=0;
mt=2*j+2;
print "M=" mt;
s=s+gj[0+1,1];
j=1;
do while j<=mt;
s=s+2*gj[j+1,1];
j=j+1;
endo;
s=s/(2*pi);
vartp = s/nmc;
"NSE and RNE=  " ;;sqrt(vartp)~nvar/vartp;


end;