/* Griddy-Gibbs for Asymmetric GJR normal-GARCH model, h0 fixed
   data: Brussels stock exchange index SPOT MARKET WEEKLY 03/01/86 - 26/01/96 

This program produces the results of Table 5 ("Normal" column) of the paper:
L. Bauwens and M. Lubrano "Bayesian inference on GARCH models using the Gibbs sampler"

It works in the same way as the program GG.PRG (which is for the results on the simulated 
data, in Table 1 of the paper). See the heading for GG.PRG for comments. Its follow-up 
program is called ANF.PRG (like GGF.PRG is the follow-up of GG.PRG).
*/

library pgraph_gnu,maxlik;
OUTPUT file = C:\Papers\Oxgauss\prg\gauss\bauwens\an1.out reset;
graphset;
fonts ("simplex complex microb simgrma");
SPATH="c:\\mydoc\\articles\\gibbs\\spotw"; @path for saving results@
datestr(0);;" at ";;timestr(0);
__title = "Spot Market 03/01/86 - 26/01/96 hebdo. (Asymmetric normal-GARCH)";
print $__title;
format 8,4;
rndseed 13579;


@ Read data @

load c[509,1] = C:\Papers\Oxgauss\prg\gauss\bauwens\spotw.dat;
cj = ln(c);
y = trimr(cj-lag1(cj),1,0);
nt = rows(y); "nt ";;nt;
@xy(seqa(1,1,nt),y);@
meany = meanc(y); stdy = stdc(y);
"EcT initial de la serie = ";;stdc(y);
y = y/stdy;
yt = y[2:nt];
y1 = y[1:nt-1];
x = yt;
x1 = y1;
t = rows(yt);
vy = vcx(yt);
@save path=^spath y;@
@"y";;y;@
"";"number of observations";;t;
"mean and s.d.";;meany~sqrt(vy);


@* Maximum likelihood estimation *@

#include maxlik.ext;
maxset;
__output = 50;
_mlparnm = "mu   "|"rho  "|"o_rt "|"a+_rt"|"b_rt "|"a-_rt";
@ some parameters are square roots of original ones, see lfGgarch: @
teta_i = 0| 0.02| 0.3| 0.3| 0.8| 0.5;
"";"teta_i";;teta_i';
{teta_h,l,s,cov,retcode} = maxprt(maxlik(yt,0,&lfGgarch,teta_i));
teta_ho = teta_h[1:2] | teta_h[3:6]^2;
jc = zeros(6,6);
jd = 1 | 1 | 2*teta_h[3:6]; @jacobian (diagonal)@
jc = diagrv(jc,jd);
covq = jc*cov*jc;
"";"MLE of natural parameters:";;
teta_se = sqrt(diag(covq));
@save path=^spath teta_ho;
save path=^spath teta_se;@
"";"teta_h se";;teta_ho~teta_se;
"";"correlation matrix";;corrvc(covq);
alphap = teta_ho[4]; beta = teta_ho[5];
alpham = teta_ho[6]; 
wsc = 1-((alphap+alpham)/2)-beta;
"";"Weak stationarity condition at MLE: ";;wsc;
" ";
"SPOT MARKET";;teta_ho';
"T=";;t~teta_se';
"wsc ";;wsc;


@* Posterior results by Gibbs sampling *@

   @@@ initialisations @@@

name ="\204m"~"\204r"~"\204w"~"\204a"~"\204b"~"\204g";@parameter names@
k = cols(name);                    @number of parameters@
teta_inf =   {-.06  .0  .01  .00001 .35  .01};  @lower bounds@
teta_sup = { .24  .4  .20  .30       .95  .60};  @upper bounds@
@save path=^spath teta_inf;
save path=^spath teta_sup;@
M=33;                              @number of points for integration@
m1 = m;                            @for trapezoidal rule in proc drawi@
w = .5 | reshape(1,m-2,1) | .5;    @weights for trapezoidal rule (see drawi)@
cdf = zeros(m1,1);                 @see proc drawi@
step = (teta_sup-teta_inf)/(m-1);  @increments@
step2 = 2*step;                    @see proc drawi@
step3 = step2/6;                   @see proc drawi@
teta_gr = zeros(m,k);              @abcissae of each parameter for integration@
i=1;
do while i<m+1;
  teta_gr[i,.] = teta_inf + (i-1)*step;
  i=i+1;
endo;
poc = zeros(m,1);                  @to store conditional posterior values@
marg = zeros(m,k);                 @to average conditionals@
f = zeros(m,k);
st = zeros(1,k);
ec = zeros(k,1);
vc = zeros(k,1);

@load teta_ho = c:\mydoc\articles\gibbs\spotw\teta_ho.fmt;@
TETA_0 = teta_ho;                  @initial conditioning values of Gibbs@
N = 2000;                          @# draws required@
NDROP = 1000;                      @# left out initial draws for final results@
NS = 1;                            @# intermediate saves in Gibbs loop@
nr = floor(n/ns);                  @# drawings before next save@
n = ns*nr;                         @actual # draws@
teta = zeros(n,k);                 @table of draws for final results@
cexp = zeros(n,k);                 @table of conditional means@
cvar = zeros(n,k);                 @table of conditional variances@
compteur = zeros(1,k);

   @@@ Gibbs loop @@@

TETA_C = teta_0;
@update teta_c if we are not in first round@
@load previous = c:\mydoc\articles\gibbs\spotw\T1050.fmt;
TETA_C = previous[rows(previous),.]'; @
"teta_c";;teta_c;
"n=";;n;
datestr(0);;" at ";;timestr(0);

js = 1;
do while js < ns+1;
  j=1;
  jo=(js-1)*nr;
  do while j<nr+1;
    i=1;
    do while i<k+1;
      f[.,i] = condi(m,poc,i,teta_gr[.,i],teta_c,yt,y1);
      @ f[.,i]: m values of posterior conditional kernel of parameter i @
      {teta_c[i],f[.,i],ec[i],vc[i]} =
      drawi(m,teta_gr[.,i],f[.,i],step[i],step2[i],step3[i],cdf,i);
      @ update teta_c with draw of parameter i from its conditional posterior @
      i=i+1;
    endo;
	
     if  j < ndrop; goto skipmar; endif;
    marg = marg + f; @adding up of (normalized) conditionals after burn-in draws @
    skipmar:    
	    
    teta[jo+j,.] = teta_c';        @assign draw j to table of draws@
    cexp[jo+j,.] = ec';            @assign cond. exp. j to table of draws@
    cvar[jo+j,.] = vc';            @assign cond. var. j to table of draws@
    @"";"j";;j;@
    @"draw";;teta_c';@
    j=j+1;
  endo;
  TIRAN1 = teta[1:js*nr,.];
  save path=^spath TIRAN1;
  TESAN1 = cexp[1:js*nr,.];
  save path=^spath TESAN1;
  TVAAN1 = cvar[1:js*nr,.];
  save path=^spath TVAAN1;
  MARAN1 = marg;
  save path=^spath MARAN1;
  "";"js-th save done";;js;
  datestr(0);;" at ";;timestr(0);
  js = js+1;
endo;
"last draw";;teta_c;

   @@@ final results @@@

pomean = meanc(teta[ndrop+1:n,.]);
povcov = vcx(teta[ndrop+1:n,.]);
pamean = meanc(cexp[ndrop+1:n,.]);
pavcov = meanc(cvar[ndrop+1:n,.]) + diag(vcx(cexp[ndrop+1:n,.]));
"";"Posterior results by Gibbs:";
"initial value and integration bounds";;teta_0~teta_inf'~teta_sup';
@ "Simspon rule with rule with ";;m;;" points"; @
"Trapezoidal rule with ";;m;;" points";
nj = n-ndrop;
"number of draws used for results";;nj;
"posterior means and st. dev.";;
pomean~sqrt(diag(povcov))~pamean~sqrt(pavcov);
"posterior correlation matrix";;corrvc(povcov);
ap = teta[ndrop+1:n,4];
bv = teta[ndrop+1:n,5];
am = teta[ndrop+1:n,6];
wcs = 1 - ((ap+am)/2) - bv;
ewcs = meanc(wcs); swcs = stdc(wcs);
pwcs = sumc(wcs.>0)/rows(wcs);
" ";
"SPOT MARKET";;pamean'~ewcs;
"T=";;t~sqrt(pavcov)'~swcs;
"P(wcs>0) = ";;pwcs;

"hit ENTER";; var=cons;
marg = marg/nj;
j=1;
do while j<k+1;
  @window(2,2,0);
  setwind(1);
  title("Histogram appr. of post. density of "$+name[j]);
  {b,mid,freq} = histp(teta[ndrop+1:n,j],17);
  setwind(3);@
  title("Posterior density of "$+name[j]);
  xy(teta_gr[.,j],marg[.,j]);
  @endwind;@
  j=j+1;
endo;
np = 17; if n-ndrop > 1000; np = 33; endif;
title("Histogram of W.S.C.");
{b,mid,freq} = histp(wcs,np);
fd = density(mid,freq);
title("Post. density of W.S.C.");
xy(mid,fd);

end;


proc DENSITY(x,h);
@* transforms histogram x (abcsissae), h (frequencies) into normalized density
   by Trapezoidal rule of integration *@
  local n,w,intg;
  n = rows(x);
  w = .5 | ones(n-2,1) | .5;
  intg = (x[2]-x[1])*w'h;
  retp(h/intg);
endp;


proc LFGGARCH(teta,x);
@* likelihood evaluation at teta for each data in x
   Normal distribution, GJR-GARCH
   result is a vector (value of LF for each row of x) *@
  local u,uq,u1,id,v;
  u = x - teta[1] - teta[2]*x1;
  uq = u[2:t]^2;
  u1 = trimr(lag1(u),1,0);
  id = u1 .< 0;
  v = recserar(teta[3]^2+(teta[4]^2*(1-id)+teta[6]^2*id).*u1^2,vy,teta[5]^2);
  retp (-0.5*(ln(v)+uq./v));
endp;


proc CONDI(m,poc,i,te,te_c,y,x1);
@* conditional posterior of parameter i evaluated at m values in te,
   conditionally on values of other parameters given in te_c
   result is a vector of m values *@
  local j;
  j=1;
  do while j<m+1;
    te_c[i] = te[j,1];
    poc[j] = pogarch(te_c,y,x1);
    j=j+1;
  endo;
  retp(poc);
endp;


proc POGARCH(teta,x,x1);
@* posterior density kernel at teta for data set x
   result is a scalar *@
  local u,uq,u1,id,v;
  u = x - teta[1] - teta[2]*x1;
  uq = u[2:t]^2;
  u1 = trimr(lag1(u),1,0);
  id = u1 .< 0;
  v = recserar(teta[3]+(teta[4]*(1-id)+teta[6]*id).*u1^2,vy,teta[5]);
  @ NB: for the next statement, the prior is proportional to 1 @
  retp (exp(sumc(-0.5*(ln(v)+uq./v))));
endp;


proc (4) = DRAWI(m,te,pdf,step,step2,step3,cdf,ii);
@* tabulate distribution function cdf at m points given in te, from density
   given in pdf at m1 points, and draws a single realization of the
   random variable by inverting the distribution numerically *@
local som,intg,ec,vc,i,u,cdfp,draw,var,inc;

@ Trapezoidal rule @
som = pdf + lag1(pdf);
som[1] = 0;
cdf = 0.5*step*cumsumc(som);
intg = cdf[m];
pdf = pdf/intg;
som = w.*pdf.*te;
ec = step*sumc(som);
vc = step*sumc(som.*te) - ec^2;

@* to check
"";"i";ii;
"tetai pdf cdf som";;te~pdf~cdf~som;
"sumc(som) step ec vc";;sumc(som)~step~ec~vc;
var = cons;
window(2,2,0);
setwind(1);
title("Conditional pdf of "$+name[ii]); xy(te,pdf);
setwind(3);
title("Conditional cdf of "$+name[ii]); xy(te,cdf);
endwind;
*@

@ draw a uniform number between 0 and cdf[m1] @
u = rndu(1,1)*intg;

@ now find between which values to interpolate @
i = indnv(1,cdf.>u);

@ and interpolate linearly @
cdfp = cdf[i-1];
draw = te[i-1] + (u-cdfp)*step/(cdf[i]-cdfp);
retp (draw,pdf,ec,vc);
endp;


proc STUDT(b,cholmi,nu,k);
@* draw of Student_k with mean b and variance nu/(nu-2)*M^{-1}
   cholmi must be equal to chol(M^{-1})' (transpose is crucial) *@

retp( b + cholmi*rndn(k,1)/sqrt(chi2(nu)/nu) );
endp;


proc CHI2(nu);
@* Algorithme dans Rubinstein page 93-94 *@
local k2,k2i;

if nu > 30;
   retp( (rndn(1,1)+sqrt(2*nu-1))^2/2 );
endif;
k2 = nu/2;
k2i = int(nu/2);
if k2 == k2i;
   retp( -2*ln(prodc(rndu(k2,1))) );
else;
   retp( -2*ln(prodc(rndu(k2,1))) + rndn(1,1)^2);
endif;
endp;
