/* Griddy-Gibbs for Asymmetric GJR Student-GARCH model, h0 fixed
   data: Brussels stock exchange index SPOT MARKET WEEKLY 03/01/86 - 26/01/96 
   Prior on nu: flat if .01 < nu < 50
   
This program produces the results for Figure 5-b ("with flat prior" curve) of the paper:
L. Bauwens and M. Lubrano "Bayesian inference on GARCH models using the Gibbs sampler"
(it also produces other results not reproduced in the paper).

It is the follow-up of ATU.PRG, exactly like GGF.PRG is the follow-up of GG.PRG (see the
comments in the heading of GGF.PRG)
*/

library pgraph,maxlik;
OUTPUT file = c:\mydoc\articles\gibbs\spotw\atuf1.out reset;
graphset;
fonts ("simplex complex microb simgrma");
SPATH="c:\\mydoc\\articles\\gibbs\\spotw"; @path for saving results@
datestr(0);;" at ";;timestr(0);
__title = "Spot Market 03/01/86 - 26/01/96 hebdo. (Asymmetric Student-GARCH)";
print $__title;
format 8,4;
rndseed 13579;


@ Read data @

load c[509,1] = c:\mydoc\articles\gibbs\spotw\spotw.dat;
cj = ln(c);
y = trimr(cj-lag1(cj),1,0);
nt = rows(y); "nt ";;nt;
@xy(seqa(1,1,nt),y);@
meany = meanc(y); stdy = stdc(y);
"EcT initial de la serie = ";;stdc(y);
y = y/stdy;
yt = y[2:nt];
y1 = y[1:nt-1];
x = yt;
x1 = y1;
t = rows(yt);
vy = vcx(yt);
@save path=^spath y;@
@"y";;y;@
"";"number of observations";;t;
"mean and s.d.";;meany~sqrt(vy);


@* Maximum likelihood estimation *@

#include maxlik.ext;
maxset;
__output = 50;
_mlparnm = "mu   "|"rho  "|"o_rt "|"a+_rt"|"b_rt "|"nu_rt"|"a-_rt";
@ some parameters are square roots of original ones, see lfGgarch: @
teta_i = 0| 0.02| 0.3| 0.3| 0.8| 2| 0.5;
"";"teta_i";;teta_i';
{teta_h,l,s,cov,retcode} = maxprt(maxlik(yt,0,&lfGgarch,teta_i));
teta_ho = teta_h[1:2] | teta_h[3:7]^2;
jc = zeros(7,7);
jd = 1 | 1 | 2*teta_h[3:7]; @jacobian (diagonal)@
jc = diagrv(jc,jd);
covq = jc*cov*jc;
"";"MLE of natural parameters:";;
teta_se = sqrt(diag(covq));
@save path=^spath teta_ho;
save path=^spath teta_se;@
"";"teta_h se";;teta_ho~teta_se;
"";"correlation matrix";;corrvc(covq);
alphap = teta_ho[4]; beta = teta_ho[5];
alpham = teta_ho[7]; nu = teta_ho[6];
wsc = 1-nu*((alphap+alpham)/2)/(nu-2)-beta;
"";"Weak stationarity condition at MLE (if nu > 2): ";;wsc;
" ";
"SPOT MARKET";;teta_ho';
"T=";;t~teta_se';
"wsc ";;wsc;


/* Posterior results by Gibbs sampling */

   @@@ initialisations @@@

name = "\204m"~"\204f"~"\204w"~"\204ap"~"\204b"~"\204n"~"\204am"; @names@
k = cols(name);                    @number of parameters@
teta_inf = {-.06  .0  .01  .00001 .35 .01 .01};  @lower bounds@
teta_sup = { .24  .4  .20  .30    .95  50 .50}; @upper bounds@
M=33;                              @number of points for marginals@
step = (teta_sup-teta_inf)/(m-1);  @increments@
teta_gr = zeros(m,k);              @abcissae of each parameter for Simpson@
i=1;
do while i<m+1;
  teta_gr[i,.] = teta_inf + (i-1)*step;
  i=i+1;
endo;
@load teta_0 = c:\mydoc\articles\gibbs\spotw\teta_ho;@
TETA_0 = teta_ho;                  @initial conditioning values of Gibbs@
NDROP = 1000;                        @# left out initial draws for final results@


   @@@ read table of draws @@@

load teta = c:\mydoc\articles\gibbs\spotw\TIRATU1;
load cexp = c:\mydoc\articles\gibbs\spotw\TESATU1;
load cvar = c:\mydoc\articles\gibbs\spotw\TVAATU1;
N = rows(teta);
"";"BAYESIAN RESULTS BY GIBBS SAMPLING:";
"";"number of rows and columns of teta:";;rows(teta)~cols(teta);


   @@@ final results @@@

pomean = meanc(teta[ndrop+1:n,.]);
povcov = vcx(teta[ndrop+1:n,.]);
posd = sqrt(diag(povcov));
pamean = meanc(cexp[ndrop+1:n,.]);
pavcov = meanc(cvar[ndrop+1:n,.]) + diag(vcx(cexp[ndrop+1:n,.]));
pasd = sqrt(pavcov);
"";"Posterior results by Gibbs:";
"initial value and integration bounds";;teta_0~teta_inf'~teta_sup';
"Trapezoidal rule with m (for densities)";;m;
nj = n-ndrop;
"Flat prior on nu";
"number of draws used for results";;nj;
"number of draws used for results";;n-ndrop;
"posterior means and st. dev.";;pomean~pamean~posd~pasd;
"posterior correlation matrix";;corrvc(povcov);
ap = teta[ndrop+1:n,4];
bv = teta[ndrop+1:n,5];
am = teta[ndrop+1:n,7];
df = teta[ndrop+1:n,6];
wcs = 1 - ((ap+am)/2).*df./(df-2) - bv;
ewcs = meanc(wcs); swcs = stdc(wcs);
pwcs = sumc(wcs.>0)/rows(wcs);
" ";
"SPOT MARKET";;pamean'~ewcs;
"T=";;t~sqrt(pavcov)'~swcs;
"P(wcs>0) = ";;pwcs;

@ alpha- - alpha+ @
adif = am-ap;
np = 17; @if n-ndrop > 1000; np = 33; endif;@
title("Histogram of adif");
{b,mid,freq} = histp(adif,np);
fd = density(mid,freq);
title("Post. density of adif");
xy(mid,fd);
"";"density of adif";mid~fd;

load marg = c:\mydoc\articles\gibbs\spotw\MARATU1;
marg = marg/nj;
"hit ENTER";; var = cons;
j=1;
do while j<k+1;
@  window(2,2,0);
  setwind(1);
  title("Histogram appr. of post. density of "$+name[j]);
  {b,mid,freq} = histp(teta[ndrop+1:n,j],17);
  setwind(3); @
  title("Posterior density of "$+name[j]);
  xy(teta_gr[.,j],marg[.,j]);
@  endwind; @
  j=j+1;
endo;

np = 17; @if n-ndrop > 1000; np = 33; endif;@
title("Histogram of W.S.C.");
{b,mid,freq} = histp(wcs,np);
fd = density(mid,freq);
title("Post. density of W.S.C.");
xy(mid,fd);
"";"density of wsc";mid~fd;

i = 1; do while i <=k;
     "";"marginale ";;i;
     teta_gr[.,i]~marg[.,i];
i = i+1; endo;


   @@@ convergence diagnostics @@@

incn = 20;
jmax = (n-ndrop)/incn;
bande = zeros(jmax,2);
bande[.,1] = 0.05*ones(jmax,1); bande[.,2] = -0.05*ones(jmax,1);
index = seqa(1,1,jmax);
exconv = zeros(jmax,k);
ecconv = zeros(jmax,k);
@sdconv = zeros(jmax,k);@
j=1;
do while j<jmax+1;
  nt = j*incn;
  nj = ndrop + nt;
  exconv[j,.] = ((meanc(teta[ndrop+1:nj,.]) - pomean)./posd)';
  ecconv[j,.] = ((meanc(cexp[ndrop+1:nj,.]) - pamean)./pasd)';
@  sdconv[j,.] = sqrt(diag(vcx(teta[ndrop+1:nj,.])))';@
  j=j+1;
endo;
nac = 20; iac = seqa(0,1,nac+1);
acf = autocorr(teta[ndrop+1:n,.],nac);
ccf = autocorr(cexp[ndrop+1:n,.],nac);
j=1;
do while j<k+1;
@  window(2,2,0);
  setwind(1); @
  title("Convergence graph of mean estimates of "$+name[j]);
  xy(index,exconv[.,j]~ecconv[.,j]);
  "";"CUMSUM series for parameter";;j;index~exconv[.,j]~ecconv[.,j]~bande;
@  setwind(2);
  title("Convergence graph of c. exp. mean of "$+name[j]);
  xy(index,ecconv[.,j]);
  title("Convergence graph of s.d. of "$+name[j]);
  xy(index,sdconv[.,j]);
  setwind(3);
  title("Convergence graph of mean/s.d. of "$+name[j]);
  xy(index,exconv[.,j]./sdconv[.,j]);
  setwind(3);
  title("Autocorrelogram of draws of "$+name[j]);
  bar(iac,acf[.,j]);
  setwind(4); @
  title("Autocorrelogram of c. exp. of "$+name[j]);
  bar(iac,ccf[.,j]);
@  endwind; @
  "";"autocorrelations of draws and c. exp. of";;j;;iac~acf[.,j]~ccf[.,j];
  j=j+1;
endo;

end;


proc LFGGARCH(teta,x);
@* likelihood evaluation at teta for each data in x
   Student distribution, GJR-GARCH
   result is a vector (value of LF for each row of x) *@
  local u,uq,u1,id,v,nu,nu1h,nuv;
  u = x - teta[1] - teta[2]*x1;
  uq = u[2:t]^2;
  u1 = trimr(lag1(u),1,0);
  id = u1 .< 0;
  v = recserar(teta[3]^2+(teta[4]^2*(1-id)+teta[7]^2*id).*u1^2,vy,teta[5]^2);
  nu = teta[6]^2; nu1h = (nu+1)/2; nuv = nu*v;
  retp ( ln(gamma(nu1h))-ln(gamma(nu/2))-0.5*ln(nuv)-nu1h*ln(1+uq./nuv) );
endp;


proc DENSITY(x,h);
@* transforms histogram x (abcsissae), h (frequencies) into normalized density
   by Simpsons's integration *@
  local n,w,intg;
  n = rows(x);
  w = .5 | ones(n-2,1) | .5;
  intg = (x[2]-x[1])*w'h;
  retp(h/intg);
endp;


PROC AUTOCORR(x,p);
  @ compute p autocorrelation coefficients for each column of x
    result in a (p+1 by cols(x) matrix) @

local n,k,j,i,xi,xl,r,ac;
n = rows(x);
k = cols(x);
xi = zeros(n,2);
ac = zeros(p+1,k);
j = 1;
do while j < k+1;
  xi[.,1] = x[.,j];
  ac[1,j] = 1;
  i = 1;
  do while i < p+1;
     xi[.,2] = lagn(xi[.,1],i);
     xl = trimr(xi,i,0);
     r = corrx(xl);
     ac[i+1,j] = r[1,2];
     i = i+1;
  endo;
  j = j+1;
endo;
retp (ac);
endp;
