/*              Griddy-Gibbs for Asymmetric GARCH-Student, h0 fixed
                             Predictive option prices 
   data: Brussels stock exchange index SPOT MARKET WEEKLY 03/01/86 - 26/01/96 

This program produces the results of Table 6 of the paper:
L. Bauwens and M. Lubrano "Bayesian inference on GARCH models using the Gibbs sampler"

This version is for an "at the money" option with 3 months maturity (correponding to 
13 weeks) and R = 1 (replication). Hence the output name is op131.out. To run for other 
maturities or more replications R, modify lines 66 and 67. To run for other options than 
"at the money", modify line 68 (set ratio different from 1), comment line 139 and remove 
the comments around lines 141-142.

This program uses the output file of griddy-Gibbs posterior draws of the parameters 
(see line 58). In this version, we use the results from ATHC.PRG */

library pgraph,maxlik;
@ output file name opSR.out see lines 66-67 for S and R @
OUTPUT file = c:\mydoc\articles\gibbs\spotw\option\op131.out reset;
graphset;
fonts ("simplex complex microb simgrma");
SPATH="c:\\mydoc\\articles\\gibbs\\spotw\\option"; @path for saving results@
datestr(0);;" at ";;timestr(0);
__title = "Spot Market 03/01/86 - 26/01/96 hebdo. (Asymmetric Student-GARCH)";
print $__title;
format 8,4;
rndseed 13579;


@* Read data *@

load c[509,1] = c:\mydoc\articles\gibbs\spotw\spotw.dat;
"last observation of c";;c[509];
cj = ln(c);
y = trimr(cj-lag1(cj),1,0);
nt = rows(y); "";"nt ";;nt;
@xy(seqa(1,1,nt),y);@
meany = meanc(y); stdy = stdc(y);
"Mean and s.d. of complete series ";;meany~stdy;
y = y/stdy;
yt = y[2:nt];
y1 = y[1:nt-1];
x = yt;
x1 = y1;
t = rows(yt);
vy = vcx(yt);
@save path=^spath y;@
@"y";;y;@
"number of observations";;t;
"Mean and s.d. ";;meany~sqrt(vy);


@* Posterior results by Gibbs sampling *@

   @@@ read table of draws @@@

load draws1 = c:\mydoc\articles\gibbs\spotw\TIRATHC1;
draws = draws1[1:2000,.];
N = rows(draws);
NDROP = 1000;                        @# left out initial draws for final results@
"";"n ";;n;


   @@@ volatility and option price @@@

S = 13; @prediction horizon@
R = 1; @number of replications@
ratio = 1; @ratio is S_T/Ke^(-rS): 1 = at the money, <1 = out, >1 = in the money@
v = zeros(t,1);
vf = zeros(S,1);
uf = zeros(S,1);
yf = zeros(S,R);
sigma = zeros(R,1);
var1 = zeros(R,1);
bs = zeros(R,1);
nd = n-ndrop;
"n-ndrop ";;nd;
ops = zeros(nd,4);

j1 = 1;
j2 = R;
vtruc = zeros(nd*R,1);

timestr(0);
j = ndrop+1;

@ LOOP ON NUMBER OF GIBBS DRAWS OF PARAMETERS 
    (for integral on theta given sample) @

do while j <= n;
@ step 1 (steps correspond to the algorithim in section 5.2 of the paper) @
   teta = draws[j,.];
   mu = teta[1];
   rho = teta[2];
   omega = teta[3];
   alphap = teta[4];
   alpham = teta[7];
   beta = teta[5];
   nu = teta[6];
   u = x - mu - rho*x1;
   u1 = trimr(lag1(u),1,0);
   id = u1 .< 0;
   v[2:t] = recserar(omega+(alphap*(1-id)+alpham*id).*u1^2,vy,beta);
@ step 2 @
   ulag = u[t];
   ind = ulag < 0;
   vf[1] = omega + alphap*ulag^2*(1-ind) + alpham*ulag^2*ind + beta*v[t];
@ step 3.1 @
   k = 1;
   do while k <=R;
@ step 3.2 @
      uf[1] = studtr(1,0,sqrt(vf[1]),nu,1);
      yf[1,k] = uf[1];
@ step 3.3.1  @
      i = 2;
      do while i<=S;
@ step 3.3.2 @
         ul = uf[i-1];
         ulc = ul^2;         
         dd = ul < 0;
         vf[i] = omega + alphap*ulc.*(1-dd) + alpham*ulc.*dd + beta*vf[i-1];
@ step 3.3.3 @
         uf[i] = studtr(1,0,sqrt(vf[i]),nu,1);
@ step 3.3.4 @
         yf[i,k] = uf[i];
@ step 3.3.5 @
      i = i+1; endo;
@ step 3.4 @

   @ Plug-in method with BS @
   mvf = meanc(vf)*nu/(nu-2); @ average of variances over horizon of option @
   sigma[k] = STDY*sqrt(mvf);
   var1[k] = mvf;
   ss = sqrt(S)*sigma[k];
   d1 = ln(ratio)/ss + .5*ss;
   bs[k] = ratio*cdfn(d1) - cdfn(d1-ss);

   @ Direct method based on max @
   ret = exp(sumc(yf*STDY));
   @ in or out the money (also at the money if ratio=1)@
   @  st = c[509]*ret;
   truc = (st/(c[509]*ratio)-1)~zeros(R,1); @
   raret = ratio*ret;
   truc = (raret-1)~zeros(R,1);
   mtruc = maxc(truc');
@ step 3.5 @
   k = k+1; endo;

@ step 4 @
   jdr = j-ndrop;
   ops[jdr,1] = meanc(sigma);
   ops[jdr,2] = meanc(bs); @ Plug-in method with BS @
   ops[jdr,3] = meanc(mtruc); @ direct method @
   ops[jdr,4] = meanc(var1); @ NEW Plug-in method with BS @
   vtruc[j1:j2] = mtruc;
   j1 = j1 + R;
   j2 = j2 + R;
j = j+1; endo;
timestr(0);


@ INTEGRAL wrt POSTERIOR (by averaging over different values, one for each drawn theta) @
mops = meanc(ops);
sops = stdc(ops);

"";"OPTION EVALUATION (BY GIBBS SAMPLING):";
"";"for post-sample horizon (maturity)";;S;
"";"number of replications";;R;
"S/K = ";;ratio;
@
"";"means and s.d. of future conditional variances";;seqa(1,1,s)~vol~svol;
@
"";"                                 Posterior exp.   s.d.";
"";"Volatility (as standard deviation)  :";;mops[1]~sops[1];
"";"Call by B-S (OLD)                   :";;mops[2]~sops[2];

@ NEW Plug-in method with BS @
sigmanew = stdy*sqrt(mops[4]);
@"sigmanew ";;sigmanew;@
ss = sqrt(S)*sigmanew;
d1 = ln(ratio)/ss + .5*ss;
bsnew = ratio*cdfn(d1) - cdfn(d1-ss);
"";"Call by B-S (NEW) and volatility    :";;bsnew~sigmanew;

"";"Call by E Max(S/K-1,0)              :";;mops[3]~sops[3];

nzer = sumc(vtruc .== 0);
"";"proportion of zero's in vtruc ";;nzer/rows(vtruc);

/*
_pdate = "";
@ window(2,2,0);
setwind(1);
title ("B-S call option price");
{b,mid1,freq1} = histp(ops[.,2],np);
setwind(2); 
title("Max call option price");
{b,mid2,freq2} = histp(ops[.,3],np);
setwind(3);
title ("B-S call option price");
f1 = density(mid1,freq1);
xy(mid1,f1);
setwind(4); 
title ("Max call option price 2");
f2 = density(mid2,freq2);
xy(mid2,f2);
endwind; @

ins = vtruc .>0;
npos = sumc(ins);
propos = npos/(nd*R);
"prop. of positive profits ";;propos;

"";"hit ENTER..."; var = cons;
posvtruc = selif(vtruc, ins);
ins1 = posvtruc .< 1;
trvtruc = selif(posvtruc,ins1);
npos1 = sumc(ins1);
"npos et npos1 ";;npos~npos1;
@np = 17;
if npos > 999; np = 33; endif;
if npos > 9999; np = 99; endif;@
np = 33;
title ("Max call option price");
{b,midv,freqv} = histp(trvtruc,np); 
fv = density(midv,freqv);
fvn = fv*propos;
xy(midv,fvn);
ta26B_in = midv~fvn;
"Call option density (continuous part)";ta26B_in;
save path = ^spath ta26B_in; 
*/

end;


proc DENSITY(x,h);
@* transforms histogram x (abcsissae), h (frequencies) into normalized density
   by Trapezoidal rule of integration *@
  local n,w,intg;
  n = rows(x);
  w = .5 | ones(n-2,1) | .5;
  intg = (x[2]-x[1])*w'h;
  retp(h/intg);
endp;


proc STUDTR (R,bi,cholimi,nui,k);
@* Tirage d'une Student de moyenne bi et de variance nu/(nu-2)*s*M^{-1}
   on fait R tirages identiques *@
  retp( bi + cholimi*rndn(k,R)/sqrt(chi2(nui)/nui) );
endp;


proc CHI2(nu);
@* Algorithme donn dans Rubinstein page 93-94 *@
  local k2,k2i;
  if nu > 30;
     retp( (rndn(1,1)+sqrt(2*nu-1))^2/2 );
  endif;
  k2 = nu/2;
  k2i = int(nu/2);
  if k2 == k2i;
     retp( -2*ln(prodc(rndu(k2,1))) );
  else;
     retp( -2*ln(prodc(rndu(k2,1))) + rndn(1,1)^2);
  endif;
endp;

/*
proc kern(x);
    local n, h, i, fx;
    x=sortc(x,1);
    n=rows(x);
    fx = zeros(n,1);
    h=2.34*stdc(x)*n^(-0.2);
    i=1;
    do while (i<=n);
        fx[i] = 1/(n*h)*sumc(quartic((x-x[i])/h));
        i=i+1;
    endo;
    fx = x~fx;
retp(fx);
endp;

proc quartic(u);
    local qk;
    qk = 15/16.*(1-u^2).*(1-u^2).*(abs(u).<1);
retp(qk);
endp;
*/